@@ -15,20 +15,27 @@
 #include <qapplication.h>
 #include <qpaintdevicemetrics.h>
 #include <qregexp.h>
+#include <kapplication.h>
+#include <klistview.h>
 #include <klocale.h>
 #include <kpassdlg.h>
+#include <kwallet.h>
 #include <kprinter.h>
 #include <ktempfile.h>
 #include <kmessagebox.h>
 #include <kdebug.h>
 
 // xpdf includes
+#include "xpdf/Object.h"
+#include "xpdf/Dict.h"
+#include "xpdf/Annot.h"
 #include "xpdf/PSOutputDev.h"
 #include "xpdf/TextOutputDev.h"
 #include "xpdf/Link.h"
 #include "xpdf/ErrorCodes.h"
 #include "xpdf/UnicodeMap.h"
 #include "xpdf/Outline.h"
+#include "xpdf/UGString.h"
 #include "goo/GList.h"
 
 // local includes
@@ -39,6 +46,8 @@
 #include "core/pagetransition.h"
 #include "conf/settings.h"
 
+#include <config.h>
+
 // id for DATA_READY PDFPixmapGeneratorThread Event
 #define TGE_DATAREADY_ID 6969
 
@@ -84,7 +93,7 @@ PDFGenerator::~PDFGenerator()
 
 
 //BEGIN Generator inherited functions
-bool PDFGenerator::loadDocument( const QString & fileName, QValueVector<KPDFPage*> & pagesVector )
+bool PDFGenerator::loadDocument( const QString & filePath, QValueVector<KPDFPage*> & pagesVector )
 {
 #ifndef NDEBUG
     if ( pdfdoc )
@@ -94,28 +103,62 @@ bool PDFGenerator::loadDocument( const QString & fileName, QValueVector<KPDFPage
     }
 #endif
     // create PDFDoc for the given file
-    pdfdoc = new PDFDoc( new GString( QFile::encodeName( fileName ) ), 0, 0 );
+    pdfdoc = new PDFDoc( new GString( QFile::encodeName( filePath ) ), 0, 0 );
 
     // if the file didn't open correctly it might be encrypted, so ask for a pass
-    bool firstTry = true;
+    bool firstInput = true;
+    bool triedWallet = false;
+    KWallet::Wallet * wallet = 0;
     while ( !pdfdoc->isOk() && pdfdoc->getErrorCode() == errEncrypted )
     {
+        QCString password;
+
+        // 1.A. try to retrieve the first password from the kde wallet system
+        if ( !triedWallet )
+        {
+            QString walletName = KWallet::Wallet::NetworkWallet();
+            wallet = KWallet::Wallet::openWallet( walletName );
+            if ( wallet )
+            {
+                // use the KPdf folder (and create if missing)
+                if ( !wallet->hasFolder( "KPdf" ) )
+                    wallet->createFolder( "KPdf" );
+                wallet->setFolder( "KPdf" );
+
+                // look for the pass in that folder
+                QString retrievedPass;
+                if ( !wallet->readPassword( filePath.section('/', -1, -1), retrievedPass ) )
+                    password = retrievedPass.local8Bit();
+            }
+            triedWallet = true;
+        }
+
+        // 1.B. if not retrieved, ask the password using the kde password dialog
+        if ( password.isNull() )
+        {
         QString prompt;
-        if ( firstTry )
+            if ( firstInput )
             prompt = i18n( "Please insert the password to read the document:" );
         else
             prompt = i18n( "Incorrect password. Try again:" );
-        firstTry = false;
+            firstInput = false;
 
-        QCString pwd;
-        if ( KPasswordDialog::getPassword( pwd, prompt ) != KPasswordDialog::Accepted )
+            // if the user presses cancel, abort opening
+            if ( KPasswordDialog::getPassword( password, prompt ) != KPasswordDialog::Accepted )
             break;
-        else
-        {
-            GString * pwd2 = new GString( pwd.data() );
+        }
+
+        // 2. reopen the document using the password
+        GString * pwd2 = new GString( password.data() );
             delete pdfdoc;
-            pdfdoc = new PDFDoc( new GString( QFile::encodeName( fileName ) ), pwd2, pwd2 );
+        pdfdoc = new PDFDoc( new GString( QFile::encodeName( filePath ) ), pwd2, pwd2 );
             delete pwd2;
+
+        // 3. if the password is correct, store it to the wallet
+        if ( pdfdoc->isOk() && wallet && /*safety check*/ wallet->isOpen() )
+        {
+            QString goodPass = QString::fromLocal8Bit( password.data() );
+            wallet->writePassword( filePath.section('/', -1, -1), goodPass );
         }
     }
     if ( !pdfdoc->isOk() )
@@ -133,8 +176,8 @@ bool PDFGenerator::loadDocument( const QString & fileName, QValueVector<KPDFPage
     pagesVector.resize( pageCount );
     for ( uint i = 0; i < pageCount ; i++ )
     {
-        KPDFPage * page = new KPDFPage( i, pdfdoc->getPageWidth(i+1),
-                                        pdfdoc->getPageHeight(i+1),
+        KPDFPage * page = new KPDFPage( i, pdfdoc->getPageCropWidth(i+1),
+                                        pdfdoc->getPageCropHeight(i+1),
                                         pdfdoc->getPageRotate(i+1) );
         addTransition( i, page );
         pagesVector[i] = page;
@@ -213,6 +256,10 @@ const DocumentSynopsis * PDFGenerator::generateDocumentSynopsis()
 
 bool PDFGenerator::isAllowed( int permissions )
 {
+#if !KPDF_FORCE_DRM
+    if (kapp->authorize("skip_drm") && !KpdfSettings::obeyDRM()) return true;
+#endif
+
     bool b = true;
     if (permissions & KPDFDocument::AllowModify) b = b && pdfdoc->okToChange();
     if (permissions & KPDFDocument::AllowCopy) b = b && pdfdoc->okToCopy();
@@ -258,26 +305,27 @@ void PDFGenerator::generatePixmap( PixmapRequest * request )
     bool genTextPage = !page->hasSearchPage() && (request->width == page->width()) &&
                        (request->height == page->height());
     // generate links and image rects if rendering pages on pageview
-    bool genObjectRects = request->id == PAGEVIEW_ID;
+    bool genObjectRects = request->id & (PAGEVIEW_ID | PRESENTATION_ID);
 
     // 0. LOCK [waits for the thread end]
     docLock.lock();
 
     // 1. Set OutputDev parameters and Generate contents
     // note: thread safety is set on 'false' for the GUI (this) thread
-    kpdfOutputDev->setParams( request->width, request->height, genTextPage, genObjectRects, genObjectRects, false );
-    pdfdoc->displayPage( kpdfOutputDev, page->number() + 1, fakeDpiX, fakeDpiY, 0, true, genObjectRects );
+    kpdfOutputDev->setParams( request->width, request->height, genObjectRects, genObjectRects, false );
+    pdfdoc->displayPage( kpdfOutputDev, page->number() + 1, fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
 
     // 2. Take data from outputdev and attach it to the Page
     page->setPixmap( request->id, kpdfOutputDev->takePixmap() );
-    if ( genTextPage )
-        page->setSearchPage( kpdfOutputDev->takeTextPage() );
     if ( genObjectRects )
         page->setObjectRects( kpdfOutputDev->takeObjectRects() );
 
     // 3. UNLOCK [re-enables shared access]
     docLock.unlock();
 
+    if ( genTextPage )
+        generateSyncTextPage( page );
+
     // update ready state
     ready = true;
 
@@ -287,12 +335,72 @@ void PDFGenerator::generatePixmap( PixmapRequest * request )
 
 void PDFGenerator::generateSyncTextPage( KPDFPage * page )
 {
-    // build a TextPage using the lightweight KPDFTextDev generator..
-    KPDFTextDev td;
+    // build a TextPage...
+    TextOutputDev td(NULL, gTrue, gFalse, gFalse);
     docLock.lock();
-    pdfdoc->displayPage( &td, page->number()+1, 72, 72, 0, true, false );
+    pdfdoc->displayPage( &td, page->number()+1, 72, 72, 0, false, true, false );
     // ..and attach it to the page
-    page->setSearchPage( td.takeTextPage() );
+    page->setSearchPage( td.takeText() );
+    docLock.unlock();
+}
+
+bool PDFGenerator::supportsSearching() const
+{
+    return true;
+}
+
+bool PDFGenerator::hasFonts() const
+{
+    return true;
+}
+
+void PDFGenerator::putFontInfo(KListView *list)
+{
+    Page *page;
+    Dict *resDict;
+    Annots *annots;
+    Object obj1, obj2;
+    int pg, i;
+
+    Ref *fonts;
+    int fontsLen;
+    int fontsSize;
+
+    list->addColumn(i18n("Name"));
+    list->addColumn(i18n("Type"));
+    list->addColumn(i18n("Embedded"));
+    list->addColumn(i18n("File"));
+
+    docLock.lock();
+
+    fonts = NULL;
+    fontsLen = fontsSize = 0;
+    for (pg = 1; pg <= pdfdoc->getNumPages(); ++pg)
+    {
+        page = pdfdoc->getCatalog()->getPage(pg);
+        if ((resDict = page->getResourceDict()))
+        {
+            scanFonts(resDict, list, &fonts, fontsLen, fontsSize);
+        }
+        annots = new Annots(pdfdoc->getXRef(), pdfdoc->getCatalog(), page->getAnnots(&obj1));
+        obj1.free();
+        for (i = 0; i < annots->getNumAnnots(); ++i)
+        {
+            if (annots->getAnnot(i)->getAppearance(&obj1)->isStream())
+            {
+                obj1.streamGetDict()->lookup("Resources", &obj2);
+                if (obj2.isDict())
+                {
+                    scanFonts(obj2.getDict(), list, &fonts, fontsLen, fontsSize);
+                }
+                obj2.free();
+            }
+            obj1.free();
+        }
+        delete annots;
+    }
+    gfree(fonts);
+
     docLock.unlock();
 }
 
@@ -340,7 +448,7 @@ bool PDFGenerator::print( KPrinter& printer )
         }
 
         docLock.lock();
-        pdfdoc->displayPages(psOut, pages, 72, 72, 0, globalParams->getPSCrop(), gFalse);
+        pdfdoc->displayPages(psOut, pages, 72, 72, 0, false, globalParams->getPSCrop(), gFalse);
         docLock.unlock();
 
         // needs to be here so that the file is flushed, do not merge with the one
@@ -356,6 +464,14 @@ bool PDFGenerator::print( KPrinter& printer )
     }
 }
 
+static UGString *QStringToUGString(const QString &s) {
+    int len = s.length();
+    Unicode *u = (Unicode *)gmallocn(s.length(), sizeof(Unicode));
+    for (int i = 0; i < len; ++i)
+      u[i] = s.at(i).unicode();
+    return new UGString(u, len);
+}
+
 QString PDFGenerator::getMetaData( const QString & key, const QString & option )
 {
     if ( key == "StartFullScreen" )
@@ -369,18 +485,12 @@ QString PDFGenerator::getMetaData( const QString & key, const QString & option )
         // asking for the page related to a 'named link destination'. the
         // option is the link name. @see addSynopsisChildren.
         DocumentViewport viewport;
-        GString * namedDest = new GString( option.latin1() );
+        UGString * namedDest = QStringToUGString( option );
         docLock.lock();
         LinkDest * destination = pdfdoc->findDest( namedDest );
         if ( destination )
         {
-            if ( !destination->isPageRef() )
-                viewport.pageNumber = destination->getPageNum() - 1;
-            else
-            {
-                Ref ref = destination->getPageRef();
-                viewport.pageNumber = pdfdoc->findPage( ref.num, ref.gen ) - 1;
-            }
+            fillViewportFromLink( viewport, destination );
         }
         docLock.unlock();
         delete namedDest;
@@ -392,7 +502,7 @@ QString PDFGenerator::getMetaData( const QString & key, const QString & option )
 
 bool PDFGenerator::reparseConfig()
 {
-    // load paper color from KpdfSettings or use the white default color
+    // load paper color from Settings or use the white default color
     QColor color = ( (KpdfSettings::renderMode() == KpdfSettings::EnumRenderMode::Paper ) &&
                      KpdfSettings::changeColors() ) ? KpdfSettings::paperColor() : Qt::white;
     // if paper color is changed we have to rebuild every visible pixmap in addition
@@ -402,7 +512,9 @@ bool PDFGenerator::reparseConfig()
     {
         paperColor = color;
         SplashColor splashCol;
-        splashCol.rgb8 = splashMakeRGB8( paperColor.red(), paperColor.green(), paperColor.blue() );
+        splashCol[0] = paperColor.red();
+	splashCol[1] = paperColor.green();
+	splashCol[2] = paperColor.blue();
         // rebuild the output device using the new paper color and initialize it
         docLock.lock();
         delete kpdfOutputDev;
@@ -425,6 +537,131 @@ static QString unicodeToQString(Unicode* u, int len) {
     return ret;
 }
 
+void PDFGenerator::scanFonts(Dict *resDict, KListView *list, Ref **fonts, int &fontsLen, int &fontsSize)
+{
+    Object obj1, obj2, xObjDict, xObj, resObj;
+    Ref r;
+    GfxFontDict *gfxFontDict;
+    GfxFont *font;
+    int i;
+
+    // scan the fonts in this resource dictionary
+    gfxFontDict = NULL;
+    resDict->lookupNF("Font", &obj1);
+    if (obj1.isRef())
+    {
+        obj1.fetch(pdfdoc->getXRef(), &obj2);
+        if (obj2.isDict())
+        {
+            r = obj1.getRef();
+            gfxFontDict = new GfxFontDict(pdfdoc->getXRef(), &r, obj2.getDict());
+        }
+        obj2.free();
+    }
+    else if (obj1.isDict())
+    {
+        gfxFontDict = new GfxFontDict(pdfdoc->getXRef(), NULL, obj1.getDict());
+    }
+    if (gfxFontDict)
+    {
+        for (i = 0; i < gfxFontDict->getNumFonts(); ++i)
+        {
+            if ((font = gfxFontDict->getFont(i))) scanFont(font, list, fonts, fontsLen, fontsSize);
+        }
+        delete gfxFontDict;
+    }
+    obj1.free();
+
+    // recursively scan any resource dictionaries in objects in this
+    // resource dictionary
+    resDict->lookup("XObject", &xObjDict);
+    if (xObjDict.isDict())
+    {
+        for (i = 0; i < xObjDict.dictGetLength(); ++i)
+        {
+            xObjDict.dictGetVal(i, &xObj);
+            if (xObj.isStream())
+            {
+                xObj.streamGetDict()->lookup("Resources", &resObj);
+                if (resObj.isDict()) scanFonts(resObj.getDict(), list, fonts, fontsLen, fontsSize);
+                resObj.free();
+            }
+            xObj.free();
+        }
+    }
+    xObjDict.free();
+}
+
+void PDFGenerator::scanFont(GfxFont *font, KListView *list, Ref **fonts, int &fontsLen, int &fontsSize)
+{
+    Ref fontRef, embRef;
+    Object fontObj, toUnicodeObj;
+    GString *name;
+    GBool emb;
+    int i;
+
+    QString fontTypeNames[8] = {
+        i18n("unknown"),
+        i18n("Type 1"),
+        i18n("Type 1C"),
+        i18n("Type 3"),
+        i18n("TrueType"),
+        i18n("CID Type 0"),
+        i18n("CID Type 0C"),
+        i18n("CID TrueType")
+    };
+
+    fontRef = *font->getID();
+
+    // check for an already-seen font
+    for (i = 0; i < fontsLen; ++i)
+    {
+        if (fontRef.num == (*fonts)[i].num && fontRef.gen == (*fonts)[i].gen)
+        {
+            return;
+        }
+    }
+
+    // font name
+    name = font->getOrigName();
+
+    // check for an embedded font
+    if (font->getType() == fontType3) emb = gTrue;
+    else emb = font->getEmbeddedFontID(&embRef);
+
+    QString sName, sEmb, sPath;
+    if (name)
+    {
+        sName = name->getCString();
+        if (!emb)
+        {
+            DisplayFontParam *dfp = globalParams->getDisplayFont(name);
+            if (dfp)
+            {
+                if (dfp -> kind == displayFontT1) sPath = dfp->t1.fileName->getCString();
+                else sPath = dfp->tt.fileName->getCString();
+            }
+            else sPath = i18n("-");
+        }
+        else sPath = i18n("-");
+    }
+    else
+    {
+        sName = i18n("[none]");
+        sPath = i18n("-");
+    }
+    sEmb = emb ? i18n("Yes") : i18n("No");
+    new KListViewItem(list, sName, fontTypeNames[font->getType()], sEmb, sPath);
+
+    // add this font to the list
+    if (fontsLen == fontsSize)
+    {
+        fontsSize += 32;
+        *fonts = (Ref *)grealloc(*fonts, fontsSize * sizeof(Ref));
+    }
+    (*fonts)[fontsLen++] = *font->getID();
+}
+
 QString PDFGenerator::getDocumentInfo( const QString & data ) const
 // note: MUTEX is LOCKED while calling this
 {
@@ -548,7 +785,7 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, GList * items )
 
         // 2. find the page the link refers to
         LinkAction * a = outlineItem->getAction();
-        if ( a && a->getKind() == actionGoTo )
+        if ( a && ( a->getKind() == actionGoTo || a->getKind() == actionGoToR ) )
         {
             // page number is contained/referenced in a LinkGoTo
             LinkGoTo * g = static_cast< LinkGoTo * >( a );
@@ -559,32 +796,21 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, GList * items )
                 // get the destination for the page now, but it's VERY time consuming,
                 // so better storing the reference and provide the viewport as metadata
                 // on demand
-                item.setAttribute( "ViewportName", g->getNamedDest()->getCString() );
+                UGString *s = g->getNamedDest();
+                QString aux = unicodeToQString( s->unicode(), s->getLength() );
+                item.setAttribute( "ViewportName", aux );
             }
             else if ( destination->isOk() )
             {
-                // we have valid 'destination' -> get page number
-                int pageNumber = destination->getPageNum() - 1;
-                if ( destination->isPageRef() )
-                {
-                    Ref ref = destination->getPageRef();
-                    pageNumber = pdfdoc->findPage( ref.num, ref.gen ) - 1;
-                }
-                // set page as attribute to node
-                // TODO add other attributes to the viewport (taken from link)
-                item.setAttribute( "Viewport", DocumentViewport( pageNumber ).toString() );
+                DocumentViewport vp;
+                fillViewportFromLink( vp, destination );
+                item.setAttribute( "Viewport", vp.toString() );
             }
-        }
-	else if ( a && a->getKind() == actionGoToR )
-        {
-            LinkGoToR * g = static_cast< LinkGoToR * >( a );
-            LinkDest * destination = g->getDest();
-            if ( !destination && g->getNamedDest() )
+            if ( a->getKind() == actionGoToR )
             {
-                item.setAttribute( "ViewportName", g->getNamedDest()->getCString() );
+                LinkGoToR * g2 = static_cast< LinkGoToR * >( a );
+                item.setAttribute( "ExternalFileName", g2->getFileName()->getCString() );
             }
-
-            item.setAttribute( "ExternalFileName", g->getFileName()->getCString() );
         }
 
         // 3. recursively descend over children
@@ -595,6 +821,51 @@ void PDFGenerator::addSynopsisChildren( QDomNode * parent, GList * items )
     }
 }
 
+void PDFGenerator::fillViewportFromLink( DocumentViewport &viewport, LinkDest *destination )
+{
+    if ( !destination->isPageRef() )
+        viewport.pageNumber = destination->getPageNum() - 1;
+    else
+    {
+        Ref ref = destination->getPageRef();
+        viewport.pageNumber = pdfdoc->findPage( ref.num, ref.gen ) - 1;
+    }
+
+    if (viewport.pageNumber == -1) return;
+
+    // get destination position
+    // TODO add other attributes to the viewport (taken from link)
+//     switch ( destination->getKind() )
+//     {
+//         case destXYZ:
+            if (destination->getChangeLeft() || destination->getChangeTop())
+            {
+                double CTM[6];
+                Page *page = pdfdoc->getCatalog()->getPage( viewport.pageNumber + 1 );
+                // TODO remember to change this if we implement DPI and/or rotation
+                page->getDefaultCTM(CTM, 72.0, 72.0, 0, gTrue);
+
+                int left, top;
+                // this is OutputDev::cvtUserToDev
+                left = (int)(CTM[0] * destination->getLeft() + CTM[2] * destination->getTop() + CTM[4] + 0.5);
+                top = (int)(CTM[1] * destination->getLeft() + CTM[3] * destination->getTop() + CTM[5] + 0.5);
+
+                viewport.rePos.normalizedX = (double)left / (double)page->getCropWidth();
+                viewport.rePos.normalizedY = (double)top / (double)page->getCropHeight();
+                viewport.rePos.enabled = true;
+                viewport.rePos.pos = DocumentViewport::TopLeft;
+            }
+            /* TODO
+            if ( dest->getChangeZoom() )
+                make zoom change*/
+/*        break;
+
+        default:
+            // implement the others cases
+        break;*/
+//     }
+}
+
 void PDFGenerator::addTransition( int pageNumber, KPDFPage * page )
 {
     Page *pdfPage = pdfdoc->getCatalog()->getPage( pageNumber + 1 );
@@ -849,16 +1120,16 @@ void PDFPixmapGeneratorThread::run()
                        ( height == page->height() );
 
     // generate links and image rects if rendering pages on pageview
-    bool genObjectRects = d->currentRequest->id == PAGEVIEW_ID;
+    bool genObjectRects = d->currentRequest->id & (PAGEVIEW_ID | PRESENTATION_ID);
 
     // 0. LOCK s[tart locking XPDF thread unsafe classes]
     d->generator->docLock.lock();
 
     // 1. set OutputDev parameters and Generate contents
-    d->generator->kpdfOutputDev->setParams( width, height, genTextPage,
+    d->generator->kpdfOutputDev->setParams( width, height, 
                                             genObjectRects, genObjectRects, TRUE /*thread safety*/ );
     d->generator->pdfdoc->displayPage( d->generator->kpdfOutputDev, page->number() + 1,
-                                       fakeDpiX, fakeDpiY, 0, true, genObjectRects );
+                                       fakeDpiX, fakeDpiY, 0, false, true, genObjectRects );
 
     // 2. grab data from the OutputDev and store it locally (note takeIMAGE)
 #ifndef NDEBUG
@@ -868,10 +1139,17 @@ void PDFPixmapGeneratorThread::run()
         kdDebug() << "PDFPixmapGeneratorThread: previous textpage not taken" << endl;
 #endif
     d->m_image = d->generator->kpdfOutputDev->takeImage();
-    d->m_textPage = d->generator->kpdfOutputDev->takeTextPage();
     d->m_rects = d->generator->kpdfOutputDev->takeObjectRects();
     d->m_rectsTaken = false;
 
+    if ( genTextPage )
+    {
+        TextOutputDev td(NULL, gTrue, gFalse, gFalse);
+        d->generator->pdfdoc->displayPage( &td, page->number()+1, 72, 72, 0, false, true, false );
+        // ..and attach it to the page
+        d->m_textPage = td.takeText();
+    }
+    
     // 3. [UNLOCK] mutex
     d->generator->docLock.unlock();
 
