@@ -36,7 +36,6 @@ class KSelectAction;
 class KAboutData;
 class KPrinter;
 
-class KPDFDocument;
 class ThumbnailList;
 class ThumbnailController;
 class PageView;
@@ -76,14 +75,22 @@ public:
 	ASYNC goToPage(uint page);
 	ASYNC openDocument(KURL doc);
 	uint pages();
+	uint currentPage();
+	KURL currentDocument();
+
+signals:
+	void enablePrintAction(bool enable);
 
 protected:
 	// reimplemented from KParts::ReadOnlyPart
 	bool openFile();
 	bool openURL(const KURL &url);
 	bool closeURL();
+  // filter that watches for splitter size changes
+  bool eventFilter( QObject * watched, QEvent * e );
 
 protected slots:
+	void openURLFromDocument(const KURL &url);
 	// connected to actions
 	void slotGoToPage();
 	void slotPreviousPage();
@@ -100,17 +107,23 @@ protected slots:
 	void slotPrintPreview();
 	void slotShowMenu(const KPDFPage *page, const QPoint &point);
 	void slotShowProperties();
+	void slotShowLeftPanel();
 	void slotShowPresentation();
+	void slotHidePresentation();
+	void close();
 	// can be connected to widget elements
 	void updateViewActions();
 	void enableTOC(bool enable);
+	void psTransformEnded();
+	void cannotQuit();
+  void saveSplitterSize();
+	void setMimeTypes(KIO::Job *job);
 
 public slots:
 	// connected to Shell action (and browserExtension), not local one
 	void slotPrint();
 	void restoreDocument(const KURL &url, int page);
 	void saveDocumentRestoreInfo(KConfig* config);
-	void slotWatchFile();
 	void slotFileDirty( const QString& );
 	void slotDoFileDirty();
 
@@ -119,6 +132,7 @@ private:
 
 	// the document
 	KPDFDocument * m_document;
+	QString m_temporaryLocalFile;
 
 	// main widgets
 	QSplitter *m_splitter;
@@ -132,6 +146,11 @@ private:
 	// static instances counter
 	static unsigned int m_count;
 
+	// this is a hack because we can not use writeConfig on part destructor
+	// and we don't want to writeconfig every time someone moves the splitter
+	// so we use a QTimer each 500 ms
+	QTimer *m_saveSplitterSizeTimer;
+
 	KDirWatch *m_watcher;
 	QTimer *m_dirtyHandler;
 	DocumentViewport m_viewportDirty;
@@ -150,8 +169,8 @@ private:
 	KAction *m_printPreview;
 	KAction *m_showProperties;
 	KAction *m_showPresentation;
-	KToggleAction* m_watchFile;
 	KToggleAction* m_showMenuBarAction;
+	KToggleAction* m_showLeftPanel;
 	KToggleAction* m_showFullScreenAction;
 	bool m_actionsSearched;
 	bool m_searchStarted;
