@@ -39,7 +39,7 @@ const PMVector upDefault = PMVector( 0.0, 0.0, 0.0 );
 const double arcAngleDefault = 0.0;
 const double falloffAngleDefault = 0.0;
 
-PMDefinePropertyClass( PMRainbow, PMProperty );
+PMDefinePropertyClass( PMRainbow, PMRainbowProperty );
 
 PMRainbow::PMRainbow( PMPart* part )
       : Base( part )
@@ -94,37 +94,37 @@ PMMetaObject* PMRainbow::metaObject( ) const
       s_pMetaObject = new PMMetaObject( "Rainbow", Base::metaObject( ),
                                         createNewRainbow );
       s_pMetaObject->addProperty(
-         new PMProperty( "direction", &PMRainbow::setDirection, &PMRainbow::direction ) );
+         new PMRainbowProperty( "direction", &PMRainbow::setDirection, &PMRainbow::direction ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "angle", &PMRainbow::setAngle, &PMRainbow::angle ) );
+         new PMRainbowProperty( "angle", &PMRainbow::setAngle, &PMRainbow::angle ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "width", &PMRainbow::setWidth, &PMRainbow::width ) );
+         new PMRainbowProperty( "width", &PMRainbow::setWidth, &PMRainbow::width ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "distance", &PMRainbow::setDistance, &PMRainbow::distance ) );
+         new PMRainbowProperty( "distance", &PMRainbow::setDistance, &PMRainbow::distance ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "jitter", &PMRainbow::setJitter, &PMRainbow::jitter ) );
+         new PMRainbowProperty( "jitter", &PMRainbow::setJitter, &PMRainbow::jitter ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "up", &PMRainbow::setUp, &PMRainbow::up ) );
+         new PMRainbowProperty( "up", &PMRainbow::setUp, &PMRainbow::up ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "arcAngle", &PMRainbow::setArcAngle, &PMRainbow::arcAngle ) );
+         new PMRainbowProperty( "arcAngle", &PMRainbow::setArcAngle, &PMRainbow::arcAngle ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "falloffAngle", &PMRainbow::setFalloffAngle, &PMRainbow::falloffAngle ) );
+         new PMRainbowProperty( "falloffAngle", &PMRainbow::setFalloffAngle, &PMRainbow::falloffAngle ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "directionEnabled", &PMRainbow::enableDirection, &PMRainbow::isDirectionEnabled ) );
+         new PMRainbowProperty( "directionEnabled", &PMRainbow::enableDirection, &PMRainbow::isDirectionEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "angleEnabled", &PMRainbow::enableAngle, &PMRainbow::isAngleEnabled ) );
+         new PMRainbowProperty( "angleEnabled", &PMRainbow::enableAngle, &PMRainbow::isAngleEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "widthEnabled", &PMRainbow::enableWidth, &PMRainbow::isWidthEnabled ) );
+         new PMRainbowProperty( "widthEnabled", &PMRainbow::enableWidth, &PMRainbow::isWidthEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "distanceEnabled", &PMRainbow::enableDistance, &PMRainbow::isDistanceEnabled ) );
+         new PMRainbowProperty( "distanceEnabled", &PMRainbow::enableDistance, &PMRainbow::isDistanceEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "jitterEnabled", &PMRainbow::enableJitter, &PMRainbow::isJitterEnabled ) );
+         new PMRainbowProperty( "jitterEnabled", &PMRainbow::enableJitter, &PMRainbow::isJitterEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "upEnabled", &PMRainbow::enableUp, &PMRainbow::isUpEnabled ) );
+         new PMRainbowProperty( "upEnabled", &PMRainbow::enableUp, &PMRainbow::isUpEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "arcAngleEnabled", &PMRainbow::enableArcAngle, &PMRainbow::isArcAngleEnabled ) );
+         new PMRainbowProperty( "arcAngleEnabled", &PMRainbow::enableArcAngle, &PMRainbow::isArcAngleEnabled ) );
       s_pMetaObject->addProperty(
-         new PMProperty( "falloffAngleEnabled", &PMRainbow::enableFalloffAngle, &PMRainbow::isFalloffAngleEnabled ) );
+         new PMRainbowProperty( "falloffAngleEnabled", &PMRainbow::enableFalloffAngle, &PMRainbow::isFalloffAngleEnabled ) );
    }
    return s_pMetaObject;
 }
