@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <stdlib.h>
@@ -144,6 +144,13 @@ void ImageWindow::setupActions()
                  this, SLOT( slotRequestPrevious() ),
                  m_actions, "previous_image" );
 
+    new KAction( i18n("Delete Image"), SHIFT + Key_Delete,
+                 this, SLOT( imageDelete() ),
+                 m_actions, "delete_image" );
+    new KAction( i18n("Move Image to Trash"), Key_Delete,
+                 this, SLOT( imageTrash() ),
+                 m_actions, "trash_image" );
+
     new KAction( i18n("Zoom In"), Key_Plus,
                  this, SLOT( zoomIn() ),
                  m_actions, "zoom_in" );
@@ -216,15 +223,15 @@ void ImageWindow::setupActions()
                  this, SLOT( scrollRight() ),
                  m_actions, "scroll_right" );
     // --------
-    KAction *pause = new KAction( i18n("Pause Slideshow"), Key_P,
-				  this, SLOT( pauseSlideShow() ),
-				  m_actions, "kuick_slideshow_pause" );
+    new KAction( i18n("Pause Slideshow"), Key_P,
+                 this, SLOT( pauseSlideShow() ),
+                 m_actions, "kuick_slideshow_pause" );
 
     KAction *fullscreenAction = KStdAction::fullScreen(this, SLOT( toggleFullscreen() ), m_actions, 0 );
 
-    new KAction( i18n("Reload Image"), Key_Enter,
-                 this, SLOT( reload() ),
-                 m_actions, "reload_image" );
+    KAction *reloadAction = new KAction( i18n("Reload Image"), KStdAccel::shortcut(KStdAccel::Reload),
+                                         this, SLOT( reload() ),
+                                         m_actions, "reload_image" );
 
     new KAction( i18n("Properties"), ALT + Key_Return,
                  this, SLOT( slotProperties() ),
@@ -234,10 +241,16 @@ void ImageWindow::setupActions()
 
     // Unfortunately there is no KAction::setShortcutDefault() :-/
     // so add Key_Return as fullscreen shortcut _after_ readShortcutSettings()
-    KShortcut cut( fullscreenAction->shortcut() );
-    if ( cut == fullscreenAction->shortcutDefault() ) {
-	cut.append(KKey(Key_Return));
-	fullscreenAction->setShortcut(cut);
+    addAlternativeShortcut(fullscreenAction, Key_Return);
+    addAlternativeShortcut(reloadAction, Key_Enter);
+}
+
+void ImageWindow::addAlternativeShortcut(KAction *action, int key)
+{
+    KShortcut cut( action->shortcut() );
+    if (cut == action->shortcutDefault()) {
+        cut.append(KKey(key));
+        action->setShortcut(cut);
     }
 }
 
@@ -501,6 +514,16 @@ void ImageWindow::lessGamma()
     addGamma( - kdata->gammaSteps );
 }
 
+void ImageWindow::imageDelete()
+{
+    emit deleteImage(this);
+}
+
+void ImageWindow::imageTrash()
+{
+    emit trashImage(this);
+}
+
 ///
 
 
@@ -534,6 +557,10 @@ void ImageWindow::keyPressEvent( QKeyEvent *e )
         close( true );
     else if ( KStdAccel::save().contains( KKey( e ) ) )
         saveImage();
+    else if ( key == Key_Right || key == Key_Down )
+        emit nextSlideRequested();
+    else if ( key == Key_Left || key == Key_Up )
+        emit prevSlideRequested(); // For future use...
 
     else {
         e->ignore();
@@ -830,6 +857,7 @@ void ImageWindow::setPopupMenu()
     viewerMenu->insertItem( i18n("Gamma"), gammaMenu );
     viewerMenu->insertSeparator();
 
+    m_actions->action("delete_image")->plug( viewerMenu );
     m_actions->action("print_image")->plug( viewerMenu );
     m_actions->action("save_image_as")->plug( viewerMenu );
     m_actions->action("properties")->plug( viewerMenu );
@@ -1098,5 +1126,4 @@ void ImageWindow::slotProperties()
     (void) new KPropertiesDialog( url, this, "props dialog", true );
 }
 
-
 #include "imagewindow.moc"
