@@ -12,8 +12,8 @@
 
    You should have received a copy of the GNU General Public License
    along with this program; see the file COPYING.  If not, write to
-   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-   Boston, MA 02111-1307, USA.
+   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
+   Boston, MA 02110-1301, USA.
 */
 
 #include <stdlib.h>
@@ -30,7 +30,8 @@ KuickData::KuickData()
   fileFilter  = "*.jpeg *.jpg *.gif *.xpm *.ppm *.pgm *.pbm *.pnm *.png *.bmp *.psd *.eim *.tif *.tiff *.xcf";// *.mng";
   slideDelay       = 3000;
   slideshowCycles  = 1;
-  slideshowFullscreen = false;
+  slideshowFullscreen = true;
+  slideshowStartAtFirst = true;
 
   preloadImage     = true;
 
@@ -57,6 +58,8 @@ KuickData::KuickData()
   maxCachedImages = 4;
   backgroundColor = Qt::black;
 
+  startInLastDir = true;
+
   idata = new ImData;
 }
 
@@ -76,7 +79,8 @@ void KuickData::load()
   fileFilter   = kc->readEntry( "FileFilter", def.fileFilter );
   slideDelay   = kc->readNumEntry( "SlideShowDelay", def.slideDelay );
   slideshowCycles = kc->readUnsignedNumEntry( "SlideshowCycles", 1 );
-  slideshowFullscreen = kc->readBoolEntry( "SlideshowFullscreen", false );
+  slideshowFullscreen = kc->readBoolEntry( "SlideshowFullscreen", true );
+  slideshowStartAtFirst = kc->readBoolEntry("SlideshowStartAtFirst", true );
 
   preloadImage = kc->readBoolEntry( "PreloadNextImage", def.preloadImage );
 
@@ -103,10 +107,12 @@ void KuickData::load()
   maxWidth 	= abs( kc->readNumEntry( "MaximumImageWidth", def.maxWidth ) );
   maxHeight 	= abs( kc->readNumEntry( "MaximumImageHeight", def.maxHeight));
 
-  maxCachedImages = kc->readUnsignedNumEntry( "MaxCachedImages", 
+  maxCachedImages = kc->readUnsignedNumEntry( "MaxCachedImages",
                                               def.maxCachedImages );
   backgroundColor = kc->readColorEntry( "BackgroundColor", &Qt::black );
 
+  startInLastDir = kc->readBoolEntry( "StartInLastDir", true);
+
   idata->load( kc );
 
   // compatibility with KuickShow <= 0.8.3
@@ -138,6 +144,7 @@ void KuickData::save()
   kc->writeEntry( "SlideShowDelay", slideDelay );
   kc->writeEntry( "SlideshowCycles", slideshowCycles );
   kc->writeEntry( "SlideshowFullscreen", slideshowFullscreen );
+  kc->writeEntry( "SlideshowStartAtFirst", slideshowStartAtFirst );
 
   kc->writeEntry( "PreloadNextImage", preloadImage );
 
@@ -166,6 +173,8 @@ void KuickData::save()
   kc->writeEntry( "MaxCachedImages", maxCachedImages );
   kc->writeEntry( "BackgroundColor", backgroundColor );
 
+  kc->writeEntry( "StartInLastDir", startInLastDir );
+
   idata->save( kc );
 
   kc->sync();
