@@ -6,7 +6,7 @@
  * them under the terms of the GNU General Public License as published by
  * the Free Software Foundation; version 2.
  */
- 
+
 
 #include <qfileinfo.h>
 #include <qtimer.h>
@@ -25,6 +25,8 @@
 #include <kstdaction.h>
 #include <kedittoolbar.h>
 #include <kurldrag.h>
+#include <kparts/partmanager.h>
+#include <kmimetype.h>
 
 #include <kprogress.h>
 #include <qlabel.h>
@@ -43,7 +45,7 @@
 #define StatusBar_ID_Zoom 3
 
 
-KViewShell::KViewShell(QString defaultMimeType)
+KViewShell::KViewShell(const QString& defaultMimeType)
   : KParts::MainWindow()
 {
   // create the viewer part
@@ -73,20 +75,18 @@ KViewShell::KViewShell(QString defaultMimeType)
   // file menu
   KStdAction::open(view, SLOT(slotFileOpen()), actionCollection());
   recent = KStdAction::openRecent (this, SLOT(openURL(const KURL &)), actionCollection());
-  reloadAction = new KAction(i18n("Reload"), "reload", CTRL + Key_R, view, SLOT(reload(void)), actionCollection(), "reload");
+  reloadAction = new KAction(i18n("Reload"), "reload", CTRL + Key_R, view, SLOT(reload()), actionCollection(), "reload");
   closeAction = KStdAction::close(this, SLOT(slotFileClose()), actionCollection());
   KStdAction::quit (this, SLOT(slotQuit()), actionCollection());
 
   connect(view, SIGNAL(fileOpened()), this, SLOT(addRecentFile()));
 
   // view menu
-#if KDE_VERSION >= KDE_MAKE_VERSION(3,1,90)
   fullScreenAction = KStdAction::fullScreen(this, SLOT(slotFullScreen()), actionCollection(), this, "fullscreen" );
 
-
   // settings menu
   createStandardStatusBarAction();
-#endif
+
   setStandardToolBarMenuEnabled(true);
 
   KStdAction::keyBindings(this, SLOT(slotConfigureKeys()), actionCollection());
@@ -119,6 +119,8 @@ KViewShell::KViewShell(QString defaultMimeType)
   statusBar()->insertFixedItem("XXXX%", StatusBar_ID_Zoom, true);
   statusBar()->changeItem("", StatusBar_ID_Zoom);
   statusBar()->insertItem(view->pageSizeDescription(), StatusBar_ID_PageSize, 0, true);
+
+  connect( view, SIGNAL(pluginChanged(KParts::Part*)), this, SLOT(createGUI(KParts::Part*)));
 }
 
 
@@ -269,7 +271,7 @@ void KViewShell::slotFullScreen()
 
 void KViewShell::slotFileClose()
 {
-  view->closeURL();
+  view->closeURL_ask();
 
   checkActions();
 }
@@ -300,15 +302,45 @@ void KViewShell::slotNewToolbarConfig()
 
 void KViewShell::dragEnterEvent(QDragEnterEvent *event)
 {
-  if (KURLDrag::canDecode(event)) 
+  if (KURLDrag::canDecode(event))
   {
     KURL::List urls;
     KURLDrag::decode(event, urls);
-    if (urls.isEmpty())
-      event->ignore();
-    else
-      event->accept();
-    //TODO: check for supported mime-types.
+    if (!urls.isEmpty())
+    {
+      KURL url = urls.first();
+
+      // Always try to open remote files
+      if (!url.isLocalFile())
+      {
+        event->accept();
+        return;
+      }
+
+      // For local files we only accept a drop, if we have a plugin for its
+      // particular mimetype
+      KMimeType::Ptr mimetype = KMimeType::findByURL(url);
+      kdDebug() << "[dragEnterEvent] Dragged URL is of type " << mimetype->comment() << endl;
+
+      // Safety check
+      if (view)
+      {
+        QStringList mimetypeList = view->supportedMimeTypes();
+        kdDebug() << "[dragEnterEvent] Supported mime types: " << mimetypeList << endl;
+
+        for (QStringList::Iterator it = mimetypeList.begin(); it != mimetypeList.end(); ++it)
+        {
+          if (mimetype->is(*it))
+          {
+            kdDebug() << "[dragEnterEvent] Found matching mimetype: " << *it << endl;
+            event->accept();
+            return;
+          }
+        }
+        kdDebug() << "[dragEnterEvent] no matching mimetype found" << endl;
+      }
+    }
+    event->ignore();
   }
 }
 
